#region File Header
// /////////////////////////////////////////////////////////////
// File: ProcessingControl.cs	Class: Kennedy.Windows.ControlsTricks.Controls.ProcessingControl
// Date: 3/20/2004				Author: Michael Kennedy
// Language: C#					Framework: .NET
//
// Copyright: Copyright (c) Michael Kennedy, 2004
// /////////////////////////////////////////////////////////////
// License: See License.txt file included with application.
// /////////////////////////////////////////////////////////////
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Kennedy.Windows.ControlsTricks.Controls
{
	public class ProcessingControl : System.Windows.Forms.UserControl
	{
		#region Member Variables

		private System.ComponentModel.IContainer components;
		
		private Kennedy.Windows.ControlsTricks.Controls.RenderLabel labelRenderTarget;
		private System.Windows.Forms.Timer timerSceneUpdate;
		private Bitmap workingImage = new Bitmap(800, 600);
		private Random random = new Random();
		private Point lastPosition = new Point(400, 300);
		private Color lastColor = Color.FromArgb(200, Color.Gray);

		#endregion

		#region Construction and Cleanup

		public ProcessingControl()
		{
			//
			// This call is required by the Windows.Forms Form Designer.
			//
			InitializeComponent();

			//
			// Add any initialization after the InitializeComponent call
			//
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (workingImage != null)
				{
					workingImage.Dispose();
					workingImage = null;
				}

				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.labelRenderTarget = new Kennedy.Windows.ControlsTricks.Controls.RenderLabel();
			this.timerSceneUpdate = new System.Windows.Forms.Timer(this.components);
			this.SuspendLayout();
			// 
			// labelRenderTarget
			// 
			this.labelRenderTarget.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.labelRenderTarget.BackColor = System.Drawing.Color.LightGray;
			this.labelRenderTarget.Location = new System.Drawing.Point(0, 0);
			this.labelRenderTarget.Name = "labelRenderTarget";
			this.labelRenderTarget.Size = new System.Drawing.Size(152, 152);
			this.labelRenderTarget.TabIndex = 0;
			this.labelRenderTarget.Text = "RENDER TARGET";
			this.labelRenderTarget.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.labelRenderTarget.Paint += new System.Windows.Forms.PaintEventHandler(this.labelRenderTarget_Paint);
			// 
			// timerSceneUpdate
			// 
			this.timerSceneUpdate.Tick += new System.EventHandler(this.timerSceneUpdate_Tick);
			// 
			// ProcessingControl
			// 
			this.Controls.Add(this.labelRenderTarget);
			this.Name = "ProcessingControl";
			this.ResumeLayout(false);

		}
		#endregion

		#region Windows Forms Event Handlers

		private void labelRenderTarget_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			PaintScene(e.Graphics);
		}

		private void timerSceneUpdate_Tick(object sender, System.EventArgs e)
		{
			UpdateScene();
		}

		#endregion

		#region Rendering and Animation Code

		public void StartAnimation()
		{
			if (timerSceneUpdate.Enabled)
			{
				return;
			}

			timerSceneUpdate.Start();
		}

		private void PaintScene(Graphics g)
		{
			g.Clear(labelRenderTarget.BackColor);

			int width = labelRenderTarget.Width;
			int height = labelRenderTarget.Height;

			lock (workingImage)
			{
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bilinear;
				g.DrawImage(workingImage, 0, 0, width, height);
			}
		}

		private void UpdateScene()
		{
			lock (workingImage)
			{	
				using (Graphics g = Graphics.FromImage(workingImage))
				{
					g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
					PaintNextStep(g);
				}
			}

			labelRenderTarget.Invalidate();
		}

		/// <summary>
		///		This is the method that does most of the work for animating the scene.
		///		The basic idea is to start at a given color and location, then randomly
		///		generate some minor change in both the color and location. Iteratively
		///		applying this idea (via the timer tick events) we get our animated sequence
		///		of lines.
		/// </summary>
		private void PaintNextStep(Graphics g)
		{
			int red = 0, green = 0, blue = 0, alpha = 0;
			bool colorInBounds = false;

			while (!colorInBounds)
			{
				alpha	= random.Next(-20, 20) + lastColor.A;
				red		= random.Next(-20, 20) + lastColor.R;
				green	= random.Next(-20, 20) + lastColor.G;
				blue	= random.Next(-20, 20) + lastColor.B;

				colorInBounds = 0 <= red   && red   < 256 &&
								0 <= green && green < 256 &&
								0 <= blue  && blue  < 256 &&
								0 <= alpha && alpha < 256;
			}

			lastColor = Color.FromArgb(200, red, green, blue);

			double angle = 0.0;
			int x = 0, y = 0, ds = 0;
			bool pointInBounds = false;
			
			while (!pointInBounds)
			{
				ds = random.Next(3, 50);
				angle = random.Next(0, 360)/(2*Math.PI);

				x = (int)Math.Round(Math.Cos(angle) * ds) + lastPosition.X;
				y = (int)Math.Round(Math.Sin(angle) * ds) + lastPosition.Y;

				if (0 < x && x < workingImage.Width && 
					0 < y && y < workingImage.Height)
				{
					pointInBounds = true;
				}
			}

			Point nextPosition = new Point(x, y);

			using (Pen pen = new Pen(lastColor, 3.0f))
			{
				g.DrawLine(pen, lastPosition, nextPosition);
				lastPosition = nextPosition;
			}
		}

		#endregion

	}
}
