#region File Header
// /////////////////////////////////////////////////////////////
// File: PopOffForm.cs	Class: Kennedy.Windows.ControlsTricks.Forms.PopOffForm
// Date: 3/20/2004		Author: Michael Kennedy
// Language: C#			Framework: .NET
//
// Copyright: Copyright (c) Michael Kennedy, 2004
// /////////////////////////////////////////////////////////////
// License: See License.txt file included with application.
// /////////////////////////////////////////////////////////////
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Kennedy.Windows.ControlsTricks.Forms
{
	public class PopOffForm : System.Windows.Forms.Form
	{
		#region Member Variables and Events

		//
		// This event is fired when this form closes. This notifies the 
		// main form that this form is just about to close. The main form
		// will then grab the render control back from this form.
		//
		public event EventHandler FormClosing;

		private System.ComponentModel.Container components = null;
		private Kennedy.Windows.ControlsTricks.Controls.ProcessingControl processControl = null;

		#endregion

		#region Construction and Cleanup

		public PopOffForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// PopOffForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(274, 224);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.Name = "PopOffForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "Pop-off Toolbox View";
			this.TopMost = true;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.PopOffForm_Closing);

		}
		#endregion

		#region Control Management Code

		public void AddControl(Kennedy.Windows.ControlsTricks.Controls.ProcessingControl processControl)
		{
			this.processControl = processControl;
			this.Controls.Add(processControl);
			processControl.Visible = true;
			processControl.Dock = DockStyle.Fill;
		}

		public void RemoveControl()
		{
			if (processControl == null)
			{
				return;
			}

			this.Controls.Remove(processControl);
			processControl = null;
		}
		
		private void PopOffForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			RemoveControl();

			if (FormClosing != null)
			{
				FormClosing(this, new EventArgs());
			}
		}

		#endregion
	}
}
