Public Class ProcessingControl
    Inherits System.Windows.Forms.UserControl

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'UserControl overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    Private WorkingImage As Bitmap = New Bitmap(800, 600)
    Private Rand As Random = New Random
    Private LastPosition As Point = New Point(400, 300)
    Private LastColor As Color = Color.FromArgb(200, Color.Gray)

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents LabelRenderTarget As Kennedy.Windows.ControlsTricks.RenderLabel
    Friend WithEvents TimerSceneUpdate As System.Windows.Forms.Timer

    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.LabelRenderTarget = New Kennedy.Windows.ControlsTricks.RenderLabel
        Me.TimerSceneUpdate = New System.Windows.Forms.Timer(Me.components)
        Me.SuspendLayout()
        '
        'LabelRenderTarget
        '
        Me.LabelRenderTarget.BackColor = System.Drawing.Color.LightGray
        Me.LabelRenderTarget.Dock = System.Windows.Forms.DockStyle.Fill
        Me.LabelRenderTarget.Location = New System.Drawing.Point(0, 0)
        Me.LabelRenderTarget.Name = "LabelRenderTarget"
        Me.LabelRenderTarget.Size = New System.Drawing.Size(140, 112)
        Me.LabelRenderTarget.TabIndex = 0
        Me.LabelRenderTarget.Text = "RENDER LABEL "
        Me.LabelRenderTarget.TextAlign = System.Drawing.ContentAlignment.MiddleCenter
        '
        'TimerSceneUpdate
        '
        '
        'ProcessingControl
        '
        Me.Controls.Add(Me.LabelRenderTarget)
        Me.Name = "ProcessingControl"
        Me.Size = New System.Drawing.Size(140, 112)
        Me.ResumeLayout(False)

    End Sub

#End Region

#Region "Windows Forms Event Handlers"

    Private Sub TimerSceneUpdate_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimerSceneUpdate.Tick

        UpdateScene()

    End Sub

    Private Sub LabelRenderTarget_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles LabelRenderTarget.Paint

        PaintScene(e.Graphics)

    End Sub

#End Region

#Region "Rendering and Animation Code"

    Private Sub UpdateScene()

        Dim G As Graphics = Graphics.FromImage(WorkingImage)
        G.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        PaintNextStep(G)

        LabelRenderTarget.Invalidate()

    End Sub

    <System.Runtime.CompilerServices.MethodImpl(Runtime.CompilerServices.MethodImplOptions.Synchronized)> _
    Private Sub PaintScene(ByVal G As Graphics)

        G.Clear(LabelRenderTarget.BackColor)

        Dim width As Integer = LabelRenderTarget.Width
        Dim height As Integer = LabelRenderTarget.Height

        G.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bilinear
        G.DrawImage(WorkingImage, 0, 0, width, height)

    End Sub

    Public Sub StartAnimation()
        If TimerSceneUpdate.Enabled Then
            Return
        End If

        TimerSceneUpdate.Start()
    End Sub

    Public Sub PaintNextStep(ByVal G As Graphics)

        Dim red As Integer = 0
        Dim green As Integer = 0
        Dim blue As Integer = 0
        Dim alpha As Integer = 0
        Dim ColorInBounds As Boolean = False

        ' Generate the current line's color randomly.
        While ColorInBounds = False

            alpha = Rand.Next(-20, 20) + LastColor.A
            red = Rand.Next(-20, 20) + LastColor.R
            green = Rand.Next(-20, 20) + LastColor.G
            blue = Rand.Next(-20, 20) + LastColor.B

            ColorInBounds = 0 <= red And red < 256 And _
                 0 <= green And green < 256 And _
                 0 <= blue And blue < 256 And _
                 0 <= alpha And alpha < 256

        End While

        LastColor = Color.FromArgb(200, red, green, blue)

        Dim angle As Double = 0.0
        Dim x As Integer = 0
        Dim y As Integer = 0
        Dim ds As Integer = 0
        Dim pointInBounds As Boolean = False

        ' Generate the next end-point randomly
        While pointInBounds = False

            ds = Rand.Next(3, 50)
            angle = Rand.Next(0, 360) / (2 * Math.PI)

            x = Math.Round(Math.Cos(angle) * ds) + LastPosition.X
            y = Math.Round(Math.Sin(angle) * ds) + LastPosition.Y

            If 0 < x And x < WorkingImage.Width And _
             0 < y And y < WorkingImage.Height Then

                pointInBounds = True

            End If

        End While

        Dim NextPosition As Point = New Point(x, y)

        ' Draw the next line using our random values.
        Dim currentPen As Pen = New Pen(LastColor, 3.0F)
        Try
            G.DrawLine(currentPen, LastPosition, NextPosition)
            LastPosition = NextPosition
        Finally
            currentPen.Dispose()
        End Try

    End Sub

#End Region

End Class
